@@ -133,10 +133,12 @@ def dag_backfill(args, dag: list[DAG] | DAG | None = None) -> None:
     """Create backfill job or dry run for a DAG or list of DAGs using regex."""
     logging.basicConfig(level=settings.LOGGING_LEVEL, format=settings.SIMPLE_LOG_FORMAT)
     signal.signal(signal.SIGTERM, sigint_handler)
-    warnings.warn(
-        "--ignore-first-depends-on-past is deprecated as the value is always set to True",
-        category=RemovedInAirflow3Warning,
-    )
+    if args.ignore_first_depends_on_past:
+        warnings.warn(
+            "--ignore-first-depends-on-past is deprecated as the value is always set to True",
+            category=RemovedInAirflow3Warning,
+        )
+    args.ignore_first_depends_on_past = True
 
     if not args.treat_dag_id_as_regex and args.treat_dag_as_regex:
         warnings.warn(
@@ -145,9 +147,6 @@ def dag_backfill(args, dag: list[DAG] | DAG | None = None) -> None:
         )
         args.treat_dag_id_as_regex = args.treat_dag_as_regex
 
-    if args.ignore_first_depends_on_past is False:
-        args.ignore_first_depends_on_past = True
-
     if not args.start_date and not args.end_date:
         raise AirflowException("Provide a start_date and/or end_date")
 
