@@ -532,8 +532,8 @@ def dag_run_link(attr):
     """Generate a URL to the Graph view for a DagRun."""
     dag_id = attr.get("dag_id")
     run_id = attr.get("run_id")
-    execution_date = attr.get("dag_run.execution_date") or attr.get("execution_date")
-    url = url_for("Airflow.graph", dag_id=dag_id, run_id=run_id, execution_date=execution_date)
+
+    url = url_for("Airflow.graph", dag_id=dag_id, dag_run_id=run_id)
     return Markup('<a href="{url}">{run_id}</a>').format(url=url, run_id=run_id)
 
 
